/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef IMPL_TYPE_PERCEPTIONBOUNDINGBOX_H
#define IMPL_TYPE_PERCEPTIONBOUNDINGBOX_H

#include <cfloat>
#include <cmath>
#include "impl_type_double.h"

struct PerceptionBoundingBox {
    ::Double confidence{ 0.0 };
    ::Double minx{ 0.0 };
    ::Double miny{ 0.0 };
    ::Double minz{ 0.0 };
    ::Double maxx{ 0.0 };
    ::Double maxy{ 0.0 };
    ::Double maxz{ 0.0 };

    static bool IsPlane()
    {
        return true;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(confidence);
        fun(minx);
        fun(miny);
        fun(minz);
        fun(maxx);
        fun(maxy);
        fun(maxz);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(confidence);
        fun(minx);
        fun(miny);
        fun(minz);
        fun(maxx);
        fun(maxy);
        fun(maxz);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("confidence", confidence);
        fun("minx", minx);
        fun("miny", miny);
        fun("minz", minz);
        fun("maxx", maxx);
        fun("maxy", maxy);
        fun("maxz", maxz);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("confidence", confidence);
        fun("minx", minx);
        fun("miny", miny);
        fun("minz", minz);
        fun("maxx", maxx);
        fun("maxy", maxy);
        fun("maxz", maxz);
    }

    friend bool operator==(const ::PerceptionBoundingBox& lhs, const ::PerceptionBoundingBox& rhs) noexcept
    {
        return (fabs(static_cast<double>(lhs.confidence - rhs.confidence)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.minx - rhs.minx)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.miny - rhs.miny)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.minz - rhs.minz)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.maxx - rhs.maxx)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.maxy - rhs.maxy)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.maxz - rhs.maxz)) < DBL_EPSILON);
    }
};


#endif // IMPL_TYPE_PERCEPTIONBOUNDINGBOX_H
